#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include "nRF24L01.h"
#include <string.h>

#define F_CPU 11059200UL
#define my(x) x*11.0592
#define BAUD 9600	  // Baud Rate
#define MYUBRR F_CPU/16/BAUD-1

#define SETBIT(x,y)  x |= (1<<y)
#define CLEARBIT(x,y) x &= ~(1<<y)

#define W 1
#define R 0

#define CE 1
#define SCN 2
#define MOSI 3
#define MISO 4
#define SCK 5
#define SPI_DDR DDRB


// UART
void USART_Init( unsigned int ubrr)
{
	/*Set baud rate */
	UBRR0H = (unsigned char)(ubrr>>8);
	UBRR0L = (unsigned char)ubrr;
	/*Enable receiver and transmitter */
	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	/* Set frame format: 8data, 1 stop bit */
	UCSR0C = (0<<USBS0)|(3<<UCSZ00);
}
void USART_Transmit( unsigned char data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) )
	;
	/* Put data into buffer, sends the data */
	UDR0 = data;
}
void print_string_USART(uint8_t *str) {
	int i;
	for (i = 0; str[i] != NULL; i++)
	USART_Transmit(str[i]);
}
//
void InitSPI() {
	SPI_DDR |= (1<<CE) | (1<<SCN) | (1<<MOSI) | (1<<SCK);
	SPCR |= (1<<SPE) | (1<<MSTR) | (1<<SPR0);
	
	
	SETBIT(PORTB, SCN);
	CLEARBIT(PORTB, CE);
}
char WriteByteSPI(char data) {
	SPDR = data;
	while (!(SPSR & (1<<SPIF)));
	return SPDR;
}
void print_reg(uint8_t reg) {
	if (((reg>>4) & 0x0f) < 10) {
		USART_Transmit('0'+((reg>>4) & 0x0f));
	}
	else {
		USART_Transmit('a'+(((reg>>4) & 0x0f)-10));
	}
	
	if ((reg & 0x0f) < 10) {
		USART_Transmit('0'+(reg & 0x0f));
	}
	else {
		USART_Transmit('a'+((reg & 0x0f)-10));
	}
	
}
void write_regs(uint8_t reg, uint8_t *data, uint8_t size) {
	uint8_t status, i;
	CLEARBIT(PORTB, SCN);
	_delay_us(my(1));
	status = WriteByteSPI(W_REGISTER + reg);
	_delay_us(my(1));
	for (i = 0; i < size; i++) {
		WriteByteSPI(data[i]);
		_delay_us(my(1));
	}
	SETBIT(PORTB, SCN);
	//print_string_USART("Status: 0x");
	//print_reg(status);
	//print_string_USART("\n");
}
uint8_t* read_regs(uint8_t reg, uint8_t size) {
	uint8_t status, i;
	uint8_t static data[5];
	CLEARBIT(PORTB, SCN);
	_delay_us(my(1));
	status = WriteByteSPI(R_REGISTER + reg);
	_delay_us(my(1));
	for (i = 0; i < size; i++) {
		data[i] = WriteByteSPI(NOP);
		_delay_us(my(1));
	}
	SETBIT(PORTB, SCN);
	//print_string_USART("Status: 0x");
	//print_reg(status);
	//print_string_USART("\n");
	return data;
}
void writePayload(uint8_t data[], uint8_t size) {
	uint8_t status, i;
	CLEARBIT(PORTB, SCN);
	_delay_us(my(1));
	status = WriteByteSPI(W_TX_PAYLOAD);
	_delay_us(my(1));
	for (i = 0; i < size; i++) {
		WriteByteSPI(data[i]);
		_delay_us(my(1));
	}
	SETBIT(PORTB, SCN);
	//print_string_USART("Status: 0x");
	//print_reg(status);
	//print_string_USART("\n");
}
uint8_t* readPayload(uint8_t size) {
	uint8_t status, i;
	uint8_t static data[32];
	CLEARBIT(PORTB, SCN);
	_delay_us(my(1));
	status = WriteByteSPI(R_RX_PAYLOAD);
	_delay_us(my(1));
	for (i = 0; i < size; i++) {
		data[i] = WriteByteSPI(NOP);
		_delay_us(my(1));
	}
	SETBIT(PORTB, SCN);
	//print_string_USART("Status: 0x");
	//print_reg(status);
	//print_string_USART("\n");
	return data;
}
void nrf24_init() {
	uint8_t *data;
	uint8_t temp;
	_delay_ms(my(15));
	
	temp = 0x0f;				// power up, receiver 
	write_regs(CONFIG, &temp, 1);
	_delay_ms(my(2));
	data = read_regs(CONFIG, 1);
	print_string_USART("CONFIG: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	temp = 0x01;				// auto acknowledge pipe 0
	write_regs(EN_AA, &temp, 1);
	data = read_regs(EN_AA, 1);
	print_string_USART("EN_AA: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	temp = 0x01;				// en rx address pipe 0
	write_regs(EN_RXADDR, &temp, 1);
	data = read_regs(EN_RXADDR, 1);
	print_string_USART("EN_RXADDR: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	temp = 0x03;				// // default retransmit
	write_regs(SETUP_RETR, &temp, 1);
	data = read_regs(SETUP_RETR, 1);
	print_string_USART("SETUP_RETR: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	temp = 0x01;				// frequency channel 1 MHz
	write_regs(RF_CH, &temp, 1);
	data = read_regs(RF_CH, 1);
	print_string_USART("RF_CH: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	temp = 0x07;				// frequency channel 1 MHz
	write_regs(RF_SETUP, &temp, 1);
	data = read_regs(RF_SETUP, 1);
	print_string_USART("RF_SETUP: 0x");
	print_reg(data[0]);
	print_string_USART("\n");
	
	data = read_regs(TX_ADDR, 5);
	print_string_USART("TX_ADDR: 0x");
	print_reg(data[0]);print_reg(data[1]);print_reg(data[2]);print_reg(data[3]);print_reg(data[4]);
	print_string_USART("\n");
	
	data = read_regs(RX_ADDR_P0, 5);
	print_string_USART("RX_ADDR_P0: 0x");
	print_reg(data[0]);print_reg(data[1]);print_reg(data[2]);print_reg(data[3]);print_reg(data[4]);
	print_string_USART("\n");
	
	_delay_us(my(10));
}
void Transmit() {
	
	uint8_t buffer[5], *read, temp;
	int i;
	for (i=0; i<5; i++) {
		buffer[i] = i;
	}
	
	read = read_regs(STATUS, 1);
	//print_string_USART("STATUS: 0x");
	//print_reg(read[0]);
	//print_string_USART("\n");
	
	read_regs(FLUSH_TX,0);
	read = read_regs(FIFO_STATUS, 1);
	//print_string_USART("FIFO_STATUS: 0x");
	//print_reg(read[0]);
	//print_string_USART("\n");
	
	print_string_USART("Sending\n");
	
	writePayload(buffer, 5);
	
	read = read_regs(FIFO_STATUS, 1);
	//print_string_USART("FIFO_STATUS: 0x");
	//print_reg(read[0]);
	//print_string_USART("\n");
	SETBIT(PORTB,CE);
	_delay_us(my(25));
	CLEARBIT(PORTB, CE);
	//_delay_ms(my(1000));
	//read = read_regs(FIFO_STATUS, 1);
	//print_string_USART("FIFO_STATUS: 0x");
	//print_reg(read[0]);
	//print_string_USART("\n");
	
	//read = read_regs(STATUS, 1);
	//print_string_USART("STATUS: 0x");
	//print_reg(read[0]);
	//print_string_USART("\n");
	
	read = read_regs(STATUS, 1);
	while ( (((read[0]>>MAX_RT) & 1) == 0) && (((read[0]>>TX_DS) & 1) == 0) ) {
		read = read_regs(STATUS, 1);
	}
	
	temp = 0x70;	// reset status reg
	write_regs(STATUS, &temp, 1);
	print_string_USART("\nSent\n");
}
void Receive() {
	uint8_t *read, temp;
	SETBIT(PORTB, CE);
	_delay_us(my(150));
	
	read = read_regs(STATUS, 1);
	while ( ((read[0]>>RX_DR) & 1) == 0) {
		read = read_regs(STATUS, 1);
	}
	
	CLEARBIT(PORTB, CE);
	
	read = readPayload(5);
	
	print_string_USART("Data received: 0x");
	print_reg(read[0]);print_reg(read[1]);print_reg(read[2]);print_reg(read[3]);print_reg(read[4]);
	print_string_USART("\n");
	
	temp = 0x70;	// reset status reg
	write_regs(STATUS, &temp, 1);
	print_string_USART("\nReceived\n");
}

int main() {
	USART_Init(MYUBRR);
	InitSPI();
	
	print_string_USART("uart\n");
	
	
	nrf24_init();
	
	
	while(1) {
		Receive();
		_delay_ms(my(1500));
	}
}

